/*
 * Copyright (c) 2018 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import (
	"bytes"
	"testing"
)

var extWhiskeySvc = Service{
	Name:         "whiskey-svc",
	ExternalName: "whiskey.example.com",
	ExternalPort: "81",
}

var extVodkaSvc = Service{
	Name:         "vodka-svc",
	ExternalName: "vodka.example.com",
	ExternalPort: "8888",
}

var barSpec = Spec{
	DefaultService: Service{},
	Rules: []Rule{
		{
			Host: "cafe.example.com",
			PathMap: map[string]Service{
				"/tea":    teaSvc,
				"/coffee": coffeeSvc,
			},
		},
		{
			Host: "whiskey.example.com",
			PathMap: map[string]Service{
				"/": extWhiskeySvc,
			},
		},
		{
			Host: "vodka.example.com",
			PathMap: map[string]Service{
				"/": extVodkaSvc,
			},
		},
	},
	IntSvcs: map[string]Service{
		"tea-svc":    teaSvc,
		"coffee-svc": coffeeSvc,
	},
	ExtSvcs: map[string]Service{
		"whiskey-svc": extWhiskeySvc,
		"vodka-svc":   extVodkaSvc,
	},
}

func TestExternalNameSvc(t *testing.T) {
	var buf bytes.Buffer
	gold := "extname.golden"
	if err := ingressTmpl.Execute(&buf, barSpec); err != nil {
		t.Fatal("Execute():", err)
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for IngressSpec does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}
